#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

setup(
    name='AIJIcossh',
    version='0.0.3',
    author='AIJI',
    author_email='thecrazyaiji@gmail.com',
    description='An ssh telecontrol Class for CentOS based on pxssh',
    packages=['cossh'],
    install_requires=['pexpect', 'AIJIdevtools'],
    url='https://github.com/AIJIJI/cossh'
                                                                                )
